// *************************************Description**************************************
/**@file LCD.c
 * @brief Functions for handling the NewHaven display
 *
 * The New Haven display uses a tile format with the b0 being in the
 * upper left corner and b7 being in column zero, 7th row.  Each column
 * is a single byte, 128 bytes across.  The next 128 bytes are for rows 8
 * through 15.  So there are eight rows for the 64 pixel vertical resolution.
 *
 * \author    chipc
 * \version   1.0
 * \date      May 27, 2019
**************************************************************************************/

/*#################################Includes##################################*/
#include "IO.h"
#include "LCD.h"
#include "Font.h"

/* #############################Private Definitions########################### */
//

/* ###############################Private Data################################ */
//

/* ###############################Private Data################################ */
//

xdata U8 frameBuffer[8][128];

/* ##############################Private Functions############################ */
//



/*******************************************************************************
 *
 * @brief
 * Converts an RLE bitmap one byte at a time
 *
 * When this function is called with a pointer to a bitmap the process of
 * converting an RLE to a bitmap begins.  With each subsequent call with a null
 * argument the function will return the next byte of pixels.
 *
 * The RLE algorithm arranges data in pairs with the first byte being the number
 * of times the data is repeated and the second byte being the data.  The first
 * count byte numbers from 1-127, with b7 reserved as a flag.  If b7 is set then
 * the count represents the number of raw data bytes that WILL NOT be repetaed, but is
 * raw data.  Since B7 is set the number is actually a negative number, such that
 * a count of 0xEC represents 20 raw data bytes shall follow, or 0x00-0xEC.
 *
 * Data is stored in a structure with data, width, height, size.
 *
 * This type of algorithm works very well with black and white images.
 *
 * @param src pointer to the bitmap payload to start, or null to extract
 *
 * @return converted data
 *
****************************************************************************** */
U8 LCDExtractRLE(U8 code *src)
{
	/* locals */
	static bit raw;
	static U8 code *rle;
	static U8 count;
	U8 ret;

	/* if the pointer is not null */
	if(src!=0)
	{
		/* start the process of extraction */
		rle=src;
		raw=0;
		count=0;

		/* return null */
		ret=0;
	}
	/* else */
	else
	{
		/* if our count is zero */
		if(count==0)
		{
			/* load a byte */
			ret=*rle++;

			/* if flagged as raw data */
			if(ret&0x80)
			{
				/* flag as raw data */
				raw=1;

				/* set our count compliment */
				count=0-ret;
			}
			/* else */
			else
			{
				/* flag as repeated */
				raw=0;

				/* set our count already returning a char */
				count=ret;
			}
		}
		/* else */
		else
		{
		}

		/* if we are raw */
		if(raw)
		{
			/* return the raw data move pointer */
			ret=*rle++;

			/* decrement the count */
			count--;
		}
		/* else */
		else
		{
			/* return the repeated data */
			ret=*rle;

			/* decrement the count */
			count--;

			/* if count just hit zero */
			if(count==0)
			{
				/* advance pointer */
				rle++;
			}
		}
	}

	/* return result */
	return ret;
}



// ###############################Public Functions############################
//

/* ******************************************************************************
 *
 * @brief
 * Writes a character bitmap from ROM to the screen
 *
 * A character bitmap is a font character formatted as tiles, similar to a font
 * file.  For this LCD a bitmap is arranged as a single byte holding a column with
 * b0 at the top.  Multiple columns are multiple bytes. Characters taller than 8
 * pixels have multiple bytes with the top byte being the low byte.  Height and
 * width are constants taken from the font file.  Only one font allowed in this
 * version.
 *
 * @param x X coordinate of placement on screen
 * @param y Y coordinate of placement on screen
 * @param src Where the bitmap is located
 *
*******************************************************************************/
void LCDWriteRLE(U8 x, U8 y, U8 code *src)
{
	/* locals */
	U8 width;
	U8 height;
	UU32 bitmap;
	U8 row;
	U8 col;
	U8 xdata *end=&frameBuffer[7][127];
	U8 xdata *dest;

	/* store the height and width */
	width=*src++;
	height=*src++;

	/* call start the RLE engine */
	LCDExtractRLE(src);

	/* for all the rows */
	for(row=0;row!=height;row++)
	{
		/* for the number of columns */
		for(col=0;col!=width;col++)
		{
			/* call get a bitmap char */
			bitmap.U32=0;
			bitmap.U8[3]=LCDExtractRLE(0);

			/* shift over the Y placement */
			bitmap.U32=bitmap.U32<<(y&0x07);

			/* calculate the byte destination */
			dest=&frameBuffer[row+(y/8)][col+x];

			/* place at this row and prepare for next row */
			*dest|=bitmap.U8[3];
			dest+=128;

			/* if not past the framebuffer end */
			if(dest<=end)
			{
				/* place at the next row */
				*dest|=bitmap.U8[2];
			}
		}
	}
}


/******************************************************************************

@brief
Writes a byte of data to the LCD

@param x byte to write


******************************************************************************/
void LCDWriteData(unsigned char x)
{
	/* call assert A0 */
	IOlcdA0(1);

	/* call assert Write line */
	IOlcdWrite(1);

	/* call place data */
	IOlcd(x);

	/* wait */
	NOP();

	/* call release Write line */
	IOlcdWrite(0);
}

/******************************************************************************

@brief
Reads a byte of data from the LCD

******************************************************************************/
unsigned char LCDReadData(void)
{
	/* locals */
	unsigned char x;

	/* call assert A0 */
	IOlcdA0(1);

	/* call assert Read line */
	IOlcdRead(1);

	/* wait */
	NOP();

	/* Fetch the data */
	x=LCDPort;

	/* call release Read line */
	IOlcdRead(0);

	/* return the data */
	return x;
}

/******************************************************************************

@brief
Writes a command to the LCD

@param x command to send

******************************************************************************/
void LCDWriteCommand(unsigned char x)
{
	/* call drop the address line */
	IOlcdA0(0);

	/* call assert Write line */
	IOlcdWrite(1);

	/* call place data */
	IOlcd(x);

	/* wait */
	NOP();

	/* call release Write line */
	IOlcdWrite(0);
}

/******************************************************************************

@brief
Writes the entire frame buffer to the display

Sends whatever is currently in the buffer to the display, buffer remains intact


******************************************************************************/
void LCDWriteBuffer(void)
{
	/* locals */
	unsigned char page=0;
	unsigned char column=0;

	/* for all pages */
	for(page=0;page<8;page++)
	{
		/* reset the column */
		column=0;

		/* call set the page and column address */
		LCDWriteCommand(LCD_PAGE_SET | page);
		LCDWriteCommand(LCD_COLUMN_SET | column);

		/* for all columns */
		for(;column<128;column++)
		{
			/* call write a character */
			LCDWriteData(frameBuffer[page][column]);
		}
	}
}

/******************************************************************************

@brief
Clears the frame buffer but not the display

******************************************************************************/
void LCDClearFrameBuffer(void)
{
	/* locals */
	unsigned char xdata *ptr=frameBuffer;
	unsigned int counter=sizeof (frameBuffer);

	/* while there are bytes to clear */
	while(counter!=0)
	{
		/* clear the byte */
		*ptr++=0;

		/* decrement the counter */
		counter--;
	}
}

/******************************************************************************

@brief
Sets the LCD column

@param x column to set

******************************************************************************/
void LCDSetColumn(unsigned char x)
{
	/* add the column to the command */
	x|=LCD_COLUMN_SET;

	/* call send the command */
	LCDWriteCommand(x);
}

/******************************************************************************

@brief
Sets the page

Sends the command to set the LCD page

@param x page to set

******************************************************************************/
void LCDSetPage(unsigned char x)
{
	/* add the row to the command */
	x|=LCD_PAGE_SET;

	/* call send the command */
	LCDWriteCommand(x);
}

/******************************************************************************

@brief
Clears the display


******************************************************************************/
void LCDClearDisplay(void)
{
	/* locals */
	unsigned char page=0;
	unsigned char column=0;

	/* for all pages */
	for(page=0;page<64;page++)
	{
		/* reset the column */
		column=0;

		/* call set the page and column address */
		LCDWriteCommand(LCD_PAGE_SET | page);
		LCDWriteCommand(LCD_COLUMN_SET | column);

		/* for all columns */
		for(;column<128;column++)
		{
			/* call write a character */
			LCDWriteData(0);
		}
	}
}

/******************************************************************************

@brief
Turns the LCD on or off

Puts the LCD to sleep or wakes it up

@param x one or zero for on or off

******************************************************************************/
void LCDDisplayRun(unsigned char x)
{
	/* set the power bit */
	x|=LCD_DISPLAY_RUN;

	/* call send the command */
	LCDWriteCommand(x);
}



/*******************************************************************************
 *
 * @brief
 * Initializes the LCD
 *
 * All values taken from the New Haven spec sheet, except for a little twiddling
 * with the contrast setting
****************************************************************************** */
void LCDInit()
{
	/* call Powerup LCD and wait */
		IOLCDOn(1);
		IOWait(2);

	/* call ADC Select Normal RAM addr */
	LCDWriteCommand(0xA0);

	/* call Common Normal Direction */
	LCDWriteCommand(0xC8);

	/* call Start Line at 0 */
	LCDWriteCommand(0x40);

	/* call wait */
	IOWaitUs(1);

	/* call set the resistor ratio */
	LCDWriteCommand(0x25);

	/* call wait */
	IOWaitUs(1);

	/* call turn on the contrast */
	LCDWriteCommand(0x81);

	/* call wait */
	IOWaitUs(1);

	/* call set the contrast */
	LCDWriteCommand(0x0E);

	/* call wait */
	IOWaitUs(1);

	/* call set the power control */
	LCDWriteCommand(0x2F);

	/* call wait */
	IOWaitUs(1);

	/* call display on */
	LCDWriteCommand(0xAF);
}

