/* Description *//**//**
 * @file Plot.c
 * @brief Functions for plotting X/Y for the tracker
 *
 * The ADC will be polled rather than ISR because that will make it a little faster
 * as far as pushing and pulling off the stack and that way the PWM will have
 * priority.
 *
 * Rev1     Feb 23, 2019  Sharp Display
 * Rev2     May 11, 2019  New Haven Display
 *
 * \author    chipc
 * \version   2.0
 * \date      May 11, 2019
 * */

/*#################################Includes##################################*/
#include "IO.h"
#include "LCD.h"
#include "keyboard.h"

// #############################Private Definitions###########################
#define CHANNEL_Y ADC0MX_ADC0MX__ADC0P14
#define CHANNEL_X ADC0MX_ADC0MX__ADC0P11
#define PERSISTANCE 5

/*###############################Private Data################################*/

/* home position of scale */
U8 mark = 4;

/*###############################Private Data################################*/

/*##############################Private Functions############################*/

/*******************************************************************************
 *
 * @brief
 * Plots the scale into the frame buffer
 *
 * The scale is a bunch of tick marks, major and minor and is moved by the keyboard
 * for centering.  The code is inline for speed.
 *
 ****************************************************************************** */
void PlotScale()
{
	/* locals */
	U8 y;
	U8 xdata *ptr;

	/* for all mark lines */
	for (y = 1; y < 8; y += 2)
	{
		/* locate the pointer */
		ptr = &frameBuffer[y][4 + mark];

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/*plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a major mark */
		*ptr = 0x0F;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a major mark */
		*ptr = 0x0F;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a major mark */
		*ptr = 0x0F;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;

		/* move the pointer */
		ptr += 8;

		/* plot a minor mark */
		*ptr = 0x03;
	}
}

/*###############################Public Functions############################*/

/*******************************************************************************
 *
 * @brief
 * brief Reads X/Y for one line cycle
 *
 * Polls the timer to complete a full line cycle.  We don't care where the cycle is
 * at the start, just as long as we capture a whole cycle and plot the results in the
 * frame buffer
 *
 *
 *
 ****************************************************************************** */
bit Plot()
{
	/* locals */
	U8 sample;
	U8 row;
	U8 page;
	static U8 persistance = 0;

	U8 x, y;
	bit ret = 0;

	/* if we should move the mark down */
	if (KeyboardGetKey() == KEY_DOWN)
	{
		/* persistance is cleared */
		persistance = 0;

		/* increment the mark */
		mark++;

		/* if the mark is too large */
		if (mark > 7)
		{
			/* reset the mark */
			mark = 0;
		}
	}

	/* if we should move the mark up */
	if (KeyboardGetKey() == KEY_UP)
	{
		/* persistance is cleared */
		persistance = 0;

		/* decrement the mark */
		mark--;

		/* if the mark is too small */
		if (mark > 7)
		{
			/* reset the mark */
			mark = 7;
		}
	}

	/* if we are still persisting */
	if (persistance != 0)
	{
		/* decrement persistance */
		persistance--;
	}
	/* else */
	else
	{
		/* call clear the buffer */
		LCDClearFrameBuffer();

		/* call PlotScale */
		PlotScale();

		/* reload persistance */
		persistance = PERSISTANCE;
	}

	/* for all voltage points */
	for (sample = 0; sample < 128; sample++)
	{
		/* wait for the timer */
		TCON_TF0 = 0;

		/*Wait for it to be time*/
		while (TCON_TF0 == 0)
			;

		/* read x */
		ADC0MX = CHANNEL_X;/*Set the channel*/

		ADC0CN0_ADINT = 0;/*Drop the flag*/
		ADC0CN0_ADBUSY = 1;/*Start the conversion*/

		/*Wait for it*/
		while (!ADC0CN0_ADINT)
			;
		x = ADC0H >> 1;

		/* read y */
		ADC0MX = CHANNEL_Y;/*Set the channel*/
		IOWaitUs(5);
		ADC0CN0_ADINT = 0;/*Drop the flag*/
		ADC0CN0_ADBUSY = 1;/*Start the conversion*/

		/*Wait for it*/
		while (!ADC0CN0_ADINT)
			;
		y = ADC0H >> 2;

		/* if this was an active read */
		if (y > 48 || y < 16)
		{
			/* flag to prevent sleep */
			ret = 1;
		}

		/* determine the page address */
		page = y / 8;

		/* determine the row address */
		row = y & 0x07;

		/* calculate the mask */
		row = 1 << row;

		/* or the pixel */
		frameBuffer[page][x] |= row;

		/* Set the mux back at x to help it settle */
		ADC0MX = CHANNEL_X;
	}

	/* call show the screen */
	LCDWriteBuffer();

	/* return TRUE if the screen saw a signal */
	return ret;
}
